package cn.com.qmth.examcloud.exchange.config;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.ThreadContext;
import org.springframework.http.HttpStatus;
import org.springframework.web.servlet.HandlerInterceptor;
import org.springframework.web.servlet.ModelAndView;

import cn.com.qmth.examcloud.api.commons.EnterpriseService;
import cn.com.qmth.examcloud.commons.logging.ExamCloudLog;
import cn.com.qmth.examcloud.commons.logging.ExamCloudLogFactory;
import cn.com.qmth.examcloud.commons.util.ByteUtil;
import cn.com.qmth.examcloud.commons.util.SHA256;
import cn.com.qmth.examcloud.commons.util.StringUtil;
import cn.com.qmth.examcloud.core.basic.api.CommonCloudService;
import cn.com.qmth.examcloud.core.basic.api.bean.ThirdPartyAccessBean;
import cn.com.qmth.examcloud.core.basic.api.request.GetThirdPartyAccessInfoReq;
import cn.com.qmth.examcloud.core.basic.api.response.GetThirdPartyAccessInfoResp;
import cn.com.qmth.examcloud.web.enums.HttpServletRequestAttribute;
import cn.com.qmth.examcloud.web.redis.RedisClient;
import cn.com.qmth.examcloud.web.support.ApiInfo;
import cn.com.qmth.examcloud.web.support.ServletUtil;
import cn.com.qmth.examcloud.web.support.StatusResponse;

/**
 * 第三方企业接入
 *
 * @author WANGWEI
 * @date 2018年5月22日
 * @Copyright (c) 2018-? http://qmth.com.cn All Rights Reserved.
 */
public final class EnterpriseAccessInterceptor implements HandlerInterceptor {

	private static final ExamCloudLog LOG = ExamCloudLogFactory
			.getLog(EnterpriseAccessInterceptor.class);

	/**
	 * 接口日志
	 */
	protected static final ExamCloudLog INTERFACE_LOG = ExamCloudLogFactory
			.getLog("INTERFACE_LOGGER");

	/**
	 * redis client
	 */
	private RedisClient redisClient;

	private CommonCloudService commonCloudService;

	/**
	 * 构造函数
	 * 
	 * @param redisClient
	 * @param commonCloudService
	 */
	public EnterpriseAccessInterceptor(RedisClient redisClient,
			CommonCloudService commonCloudService) {
		super();
		this.redisClient = redisClient;
		this.commonCloudService = commonCloudService;
	}

	@Override
	public boolean preHandle(HttpServletRequest request, HttpServletResponse response,
			Object handler) throws Exception {
		LOG.debug("preHandle... ...");

		ApiInfo apiInfo = (ApiInfo) request
				.getAttribute(HttpServletRequestAttribute.$_API_INFO.name());

		if (null != apiInfo) {
			Class<?> ctrClass = apiInfo.getBeanType();
			if (!EnterpriseService.class.isAssignableFrom(ctrClass)) {
				return true;
			}
		} else {
			return true;
		}

		String appId = request.getHeader("App-Id");
		if (null == appId) {
			appId = request.getHeader("appId");
		}
		if (StringUtils.isBlank(appId)) {
			// 403
			response.setStatus(HttpStatus.FORBIDDEN.value());
			ServletUtil.returnJson(new StatusResponse("403", "'App-Id'('appId') is blank"),
					response);
			return false;
		}

		String rootOrgId = request.getHeader("Root-Org-Id");
		if (null == rootOrgId) {
			rootOrgId = request.getHeader("rootOrgId");
		}
		if (StringUtils.isBlank(rootOrgId)) {
			// 403
			response.setStatus(HttpStatus.FORBIDDEN.value());
			ServletUtil.returnJson(new StatusResponse("403", "'Root-Org-Id'('rootOrgId') is blank"),
					response);
			return false;
		}
		Long rootOrgIdLong = null;
		try {
			rootOrgIdLong = Long.parseLong(rootOrgId);
		} catch (Exception e) {
			// 403
			response.setStatus(HttpStatus.FORBIDDEN.value());
			ServletUtil.returnJson(
					new StatusResponse("403", "'Root-Org-Id'('rootOrgId') must be a long"),
					response);
			return false;
		}

		String accessToken = request.getHeader("Access-Token");
		if (null == accessToken) {
			accessToken = request.getHeader("access_token");
		}
		if (StringUtils.isBlank(accessToken)) {
			// 403
			response.setStatus(HttpStatus.FORBIDDEN.value());
			ServletUtil.returnJson(
					new StatusResponse("403", "'Access-Token'('access_token') is blank"), response);
			return false;
		}

		String timestamp = request.getHeader("timestamp");
		if (StringUtils.isBlank(timestamp)) {
			// 403
			response.setStatus(HttpStatus.FORBIDDEN.value());
			ServletUtil.returnJson(new StatusResponse("403", "'timestamp' is blank"), response);
			return false;
		}
		Long timestampLong = null;
		try {
			timestampLong = Long.parseLong(timestamp);
		} catch (Exception e) {
			// 403
			response.setStatus(HttpStatus.FORBIDDEN.value());
			ServletUtil.returnJson(new StatusResponse("403", "'timestamp' must be a long"),
					response);
			return false;
		}

		request.setAttribute(HttpServletRequestAttribute.$_ENTERPRISE_ROOT_ORG_ID.name(),
				rootOrgIdLong);

		String key = "$_A_:" + rootOrgId + "_" + appId;

		ThirdPartyAccessBean thirdPartyAccessBean = redisClient.get(key,
				ThirdPartyAccessBean.class);

		if (null == thirdPartyAccessBean) {
			thirdPartyAccessBean = getThirdPartyAccessInfo(rootOrgIdLong, appId);
			redisClient.set(key, thirdPartyAccessBean, 60 * 5);
		}

		if (null != thirdPartyAccessBean.getTimeRange()) {
			long currentTimeMillis = System.currentTimeMillis();
			if (Math.abs(currentTimeMillis - timestampLong) > thirdPartyAccessBean.getTimeRange()) {
				// 403
				response.setStatus(HttpStatus.FORBIDDEN.value());
				ServletUtil.returnJson(new StatusResponse("403", "timestamp is out"), response);
				return false;
			}
		}

		String secretKey = thirdPartyAccessBean.getSecretKey();
		String joinStr = StringUtil.join(rootOrgId, appId, timestamp, secretKey);
		byte[] bytes = SHA256.encode(joinStr);
		String hexAscii = ByteUtil.toHexAscii(bytes);

		if (!hexAscii.equalsIgnoreCase(accessToken)) {
			// 403
			response.setStatus(HttpStatus.FORBIDDEN.value());
			ServletUtil.returnJson(
					new StatusResponse("403", "'Access-Token'('access_token') is wrong"), response);
			return false;
		}

		ThreadContext.put("CALLER", "ACCESS:" + rootOrgIdLong);

		return true;
	}

	private ThirdPartyAccessBean getThirdPartyAccessInfo(Long rootOrgId, String appId) {
		ThirdPartyAccessBean thirdPartyAccessBean;
		GetThirdPartyAccessInfoReq req = new GetThirdPartyAccessInfoReq();
		req.setAppId(appId);
		req.setRootOrgId(rootOrgId);
		GetThirdPartyAccessInfoResp resp = commonCloudService.getThirdPartyAccessInfo(req);
		thirdPartyAccessBean = resp.getThirdPartyAccessBean();
		return thirdPartyAccessBean;
	}

	@Override
	public void postHandle(HttpServletRequest request, HttpServletResponse response, Object handler,
			ModelAndView modelAndView) throws Exception {
		LOG.debug("postHandle... ...");
	}

	@Override
	public void afterCompletion(HttpServletRequest request, HttpServletResponse response,
			Object handler, Exception ex) throws Exception {
		LOG.debug("afterCompletion... ...");
	}

}
